require "spec_helper"

module EventTestWorkflow
  module OddWorkflow
    def self.run(id)
      raise "this is for the odd" unless id.odd?
    end

    Postjob.register_workflow self
  end

  def self.run
    await OddWorkflow, 1
    await OddWorkflow, 2
  end

  Postjob.register_workflow self
end

describe "Job Events" do
  include TestHelper

  let!(:job) do
    id = Postjob.enqueue! "EventTestWorkflow"
    load_job id
  end

  before do
    TestHelper.resolve_all
  end

  def job_events(*jobs)
    events_query = "SELECT * FROM postjob.events WHERE postjob_id = ANY($1) ORDER BY created_at"
    Simple::SQL.all events_query, jobs.map(&:id), into: OpenStruct
  end

  it "creates events on each job status change" do
    good_child = load_job("SELECT * FROM postjob.postjobs WHERE parent_id=#{job.id} ORDER BY id")
    bad_child  = load_job("SELECT * FROM postjob.postjobs WHERE parent_id=#{job.id} ORDER BY id OFFSET 1")

    expect(job_events(good_child).map(&:name)).to eq(["ready", "processing", "ok"])

    bad_child_events = %w(
      ready
      processing err
      processing err
      processing err
      processing err
      processing failed
    )
    expect(job_events(bad_child).map(&:name)).to eq(bad_child_events)

    toplevel_events = %w(
      ready
      processing sleep ready
      processing sleep ready
      processing sleep ready
      processing sleep ready
      processing sleep ready
      processing sleep ready
      processing failed
    )
    expect(job_events(job).map(&:name)).to eq(toplevel_events)
  end

  describe "automatic event creation" do
    let(:jobs) { Simple::SQL.all("SELECT * FROM postjob.postjobs WHERE root_id=#{job.id}", into: Postjob::Job) }
    let(:events) { job_events(*jobs) }

    it "sets the job's last_worker_session_id" do
      last_worker_session_ids = jobs.map(&:last_worker_session_id)
      expect(last_worker_session_ids.uniq).to eq([Postjob.current_session_id])
    end

    it "records the worker session id" do
      worker_session_ids = events.map(&:worker_session_id).uniq
      expect(worker_session_ids).to eq([Postjob.current_session_id])
    end
  end
end
