# rubocop:disable Style/MethodMissing

require "spec_helper"

#
# This module simulates the MPX services interfaces, with a certain
# change not to be able to run (see operational_error?)
module MPX
  module Impl
    extend self

    def self.load_users(user_ids)
      user_ids.map do |user_id|
        {
          "type" => "User",
          "id" => user_id
        }
      end
    end

    def self.clone_group(group_id)
      group_id + 1000
    end

    def self.send_email(_user, _cloned_group_id)
      "sent"
    end
  end

  def self.method_missing(sym, *args)
    operational_error?
    Impl.send sym, *args
  end

  # The operational_error method introduces a certain chance of not being able
  # to run one of the MPX.* methods, simulating a network timeout or other such
  # error condition.
  def self.operational_error?
    raise "please rerun" if rand < 0.1
  end
end

module RecommendGroupWorkflow
  Postjob.register_workflow self

  def self.clone_group(group_id)
    MPX.clone_group(group_id)
  end

  def self.load_users(user_ids)
    MPX.load_users(user_ids)
  end

  def self.send_email(user, cloned_group_id)
    MPX.send_email user, cloned_group_id
  end

  def self.run(group_id, user_ids)
    users = await :load_users, user_ids

    cloned_group_id = await :clone_group, group_id
    users.each do |user|
      async :send_email, user, cloned_group_id
    end
    await :all
    users.count
  end
end

describe "Sub Workflows" do
  include TestHelper

  let!(:users_count) { 100 }
  let!(:user_ids) { 100.upto(100 + users_count - 1).to_a }
  let!(:id) { Postjob.enqueue! "RecommendGroupWorkflow", 1, user_ids }

  xit "runs the job returning the result" do
    expect(MPX::Impl).to receive(:clone_group).exactly(1).times.and_call_original
    expect(MPX::Impl).to receive(:load_users).exactly(1).times.and_call_original
    expect(MPX::Impl).to receive(:send_email).exactly(users_count).times.and_call_original

    Postjob.process_all

    processed_job = load_job(id)
    expect(processed_job.status).to eq("ok")
  end
end
