require "spec_helper"

RSpec.describe Postjob::Host do
  UUID_REGEXP = /[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/i

  describe ".host_id" do
    it "returns a host id UUID string" do
      expect(Postjob::Host.host_id).to match(UUID_REGEXP)
    end
  end

  # [TODO] - this spec might have to be rewritten. We now start a session before
  #          we can mock the storage_path, therefore the host id will already be
  #          written to the "real" place.
  xdescribe "Storage" do
    let(:test_file_path) { ".postjob.test.host_id" }

    before do
      allow(Postjob::Host).to receive(:storage_path).and_return(test_file_path)
    end

    describe ".clear_storage" do
      it "removes the host id file" do
        Postjob::Host.clear_storage

        expect(File).not_to exist(test_file_path)
      end
    end

    describe ".host_id" do
      it "writes a file to the tmp directory" do
        Postjob::Host.host_id

        expect(File).to exist(test_file_path)
      end
    end
  end
end
