require "spec_helper"

# DO NOT TRY THIS IN THE REAL WORLD!
#
# This workflow uses a global value, FailureWorkflow.attempts, to count the
# number of attempts to run a workflow. This turns the workflow into a stateful
# affair WHICH BREAKS THE GENERAL CONTRACT OF POSTJOB.
module FailureWorkflow
  extend self

  attr_accessor :attempts

  def run(number_of_failing_attempts)
    self.attempts += 1
    if self.attempts <= number_of_failing_attempts
      raise "Failure/Workflow failed"
    end

    "result"
  end

  Postjob.register_workflow self
end

describe "error states" do
  before do
    FailureWorkflow.attempts = 0
  end

  def load_job
    TestHelper.load_job(job_id)
  end

  let!(:job_id) { Postjob.enqueue!("FailureWorkflow", 1) }

  context "when running a job that runs into an exception" do
    before do
      Postjob.process_all
    end

    it "sets the job's status to err" do
      expect(load_job.status).to eq("err")
    end

    it "sets the job's error information" do
      expect(load_job.error_message).to eq("Failure/Workflow failed")
      expect(load_job.error).to eq("RuntimeError")
      expect(load_job.error_backtrace).to be_a(Array)
    end
  end

  context "when rerunning the job so that it succeeds" do
    before do
      sleep 0.1 while Postjob.process_all > 0
    end

    it "sets the job's status to ok" do
      expect(load_job.status).to eq("ok")
    end

    it "sets the job's result" do
      expect(load_job.result).to eq("result")
    end

    it "cleared out the error entries" do
      expect(load_job.error_message).to be_nil
      expect(load_job.error).to be_nil
      expect(load_job.error_backtrace).to be_nil
    end
  end
end
