require "spec_helper"

module ManualWorkflow
  def self.run(max_attempts)
    job = async :manual, timeout: 10, max_attempts: max_attempts
    _token = workflow_token job
    manual_result = await job
    "manual-result:#{manual_result}"
  end

  Postjob.register_workflow self
end

describe "manual processing" do
  let!(:id) { Postjob.enqueue! "ManualWorkflow", 3 }

  include TestHelper

  before do
    Postjob.process_all
  end

  def load_child_job
    TestHelper.load_job "SELECT * FROM postjobs WHERE parent_id=$1", id
  end

  it "sets max_attempts to 1" do
    expect(load_child_job[:max_attempts]).to eq(1)
  end

  it "creates a UUID token" do
    token = load_token(load_child_job)
    expect! token => /[A-F0-9]{8}-[A-F0-9]{4}-[A-F0-9]{4}-[a-fA-F0-9]{4}-[A-F0-9]{12}/i
  end

  it "the token can be used to resolve the child job" do
    token = load_token(load_child_job)
    Postjob.resolve token: token, result: "foobar"
    expect(load_child_job.status).to eq("ok")
    expect(load_child_job.result).to eq("foobar")

    Postjob.process_all
    job = load_job id
    expect(job.status).to eq("ok")
    expect(job.result).to eq("manual-result:foobar")
  end
end
