require "spec_helper"

module MaxAttemptWorkflow
  module ChildWorker
    def self.run
      raise "Nono nono!"
    end

    Postjob.register_workflow self
  end

  def self.run(max_attempts)
    await async ChildWorker, max_attempts: max_attempts
  end

  Postjob.register_workflow self
end

describe "max_attempts" do
  before do
    Simple::SQL.all("DELETE FROM postjob.postjobs")
  end

  def load_child_job
    TestHelper.load_job "SELECT * FROM postjobs WHERE parent_id=$1", id
  end

  def load_job
    TestHelper.load_job(id)
  end

  def load_child_job
    TestHelper.load_job "SELECT * FROM postjobs WHERE parent_id=$1", id
  end

  context "when running with max_attempts 1" do
    let!(:id) { Postjob.enqueue!("MaxAttemptWorkflow", 1, max_attempts: 1) }

    it "fails the childjob with a timeout" do
      sleep 0.03 while Postjob.process_all > 0

      expect(load_child_job.failed_attempts).to eq(1)
      expect(load_child_job.status).to eq("failed")

      expect(load_job.status).to eq("failed")
    end
  end

  context "when running with max_attempts 5" do
    let!(:id) { Postjob.enqueue!("MaxAttemptWorkflow", 5, max_attempts: 1) }

    it "fails the childjob with a timeout" do
      while Postjob.process_all > 0
        sleep 0.1
      end

      expect(load_child_job.failed_attempts).to eq(5)
      expect(load_child_job.status).to eq("failed")

      # The main job now failed. It only ran with max_attempts: 1
      expect(load_job.status).to eq("failed")
    end
  end
end
