require "spec_helper"

module EncodeSpecWorkflow
  def self.run(_inp, out)
    out
  end

  Postjob.register_workflow self
end

describe "Postjob::Queue::Encoder" do
  def run_workflow(input: "dummy input", output: "dummy output")
    id = Postjob.enqueue! "EncodeSpecWorkflow", input, output
    Postjob.process_all
    job = TestHelper.load_job(id)
    job.result
  end

  it "encodes nil into nil" do
    expect(run_workflow(input: nil, output: nil)).to eq(nil)
  end

  it "allows encoding of numbers" do
    expect(run_workflow(input: 10, output: 12)).to eq(12)
  end

  it "allows encoding of booleans" do
    expect(run_workflow(input: true, output: false)).to eq(false)
  end

  it "allows encoding of strings" do
    expect(run_workflow(input: "foo", output: "bar")).to eq("bar")
  end

  it "allows encoding of arrays" do
    expect(run_workflow(input: ["foo", "foo"], output: ["bar", "baz"])).to eq(["bar", "baz"])
  end

  describe "invalid input" do
    def self.check_value(value)
      it "cannot encode input" do
        expect do
          run_workflow input: value
        end.to raise_error(::Postjob::Error::Encoding)
      end

      it "cannot encode output" do
        expect do
          run_workflow output: value
        end.to raise_error(::Postjob::Error::Encoding)
      end
    end

    context "when encoding symbols" do
      check_value :foo
    end

    context "when encoding invalid utf8 strings" do
      check_value 255.chr.force_encoding("utf-8")
      check_value 255.chr
    end
  end
end
