require "spec_helper"

module TwoLevelWorkflow
  module StringWorker
    def self.run
      "my string"
    end

    Postjob.register_workflow self
  end

  def self.run
    r = async StringWorker
    await r
  end

  Postjob.register_workflow self
end

describe "Postjob.process_all" do
  include TestHelper

  let!(:id) { Postjob.enqueue! "TwoLevelWorkflow" }

  it "runs the job returning the result" do
    expect(Postjob.process_all).to eq(3)
    processed_job = load_job(id)
    expect(processed_job.status).to eq("ok")
    expect(processed_job.resolve).to eq("my string")
  end

  context "when scheduled to run in the future" do
    before do
      Simple::SQL.ask <<~SQL, id
        UPDATE postjob.postjobs
        SET next_run_at=next_run_at + interval '10 seconds'
        WHERE id=$1
        SQL
    end

    it "does not run the job" do
      expect(Postjob.process_all).to eq(0)
      processed_job = load_job(id)
      # expect(processed_job.status).to eq("ready")
      expect(processed_job.resolve).to eq(:pending)
    end
  end

  context "when scheduled to time out" do
    before do
      Simple::SQL.ask <<~SQL, id
        UPDATE postjob.postjobs
        SET timing_out_at=next_run_at
        WHERE id=$1
        SQL
    end

    it "times out when job is running" do
      Postjob.process_all
      processed_job = load_job(id)

      expect do
        processed_job.resolve
      end.to raise_error(Timeout::Error)
    end
  end
end
