require "spec_helper"

module ZombieSpecWorkflow
  Postjob.register_workflow self
end

describe "Zombie Detection" do
  include TestHelper

  let(:host_id) { @worker_session.host_id }

  def send_heartbeat!
    # pass in a artificial heartbeat
    ::Postjob::Queue.host_heartbeat(host_id, {})
  end

  context "with a processing job which has the latest heartbeat from its host for > 5 minute" do
    before do
      @job_id = Postjob.enqueue! "ZombieSpecWorkflow"
      SQL.ask "UPDATE postjob.postjobs SET status='processing'"
      send_heartbeat!
      SQL.ask "UPDATE postjob.events SET created_at = (now() at time zone 'utc' - interval '6 minutes')"
    end

    context "when sending a heartbeat" do
      before do
        send_heartbeat!
      end

      it "changes the job to zombie" do
        job = SQL.ask "SELECT * FROM postjob.postjobs WHERE id= #{@job_id}", into: Postjob::Job

        expect(job.error).to eq("Zombie")
        expect(job.error_message).to match("host .* disappeared")
        expect(job.status).to eq("err")
      end

      it "creates a zombie event with a zombie_count" do
        event = SQL.ask "SELECT * FROM postjob.events WHERE name='zombie' ORDER BY created_at DESC LIMIT 1", into: OpenStruct
        expect(event.name).to eq("zombie")
        expect(event.attributes).to eq("zombie_count" => 1)
      end
    end
  end
end
