ENV["POSTJOB_ENV"] = "test"

require "rspec"
require "pry"
require "simplecov"
require "awesome_print"

unless ENV["SKIP_SIMPLE_COV"]
  SimpleCov.start do
    # minimum_coverage 86
    add_filter "/spec/"
  end
end

require "postjob"
Postjob.fast_mode = true

logger = Logger.new("log/test.log")
logger.formatter = proc do |severity, _datetime, _progname, msg|
  "#{severity}: #{msg}\n"
end
Simple::SQL.logger = Postjob.logger = logger

require "./spec/support/configure_database"
require "./spec/support/test_helper"

if ENV["VERBOSE"]
  logger = Logger.new(STDERR)
  Simple::SQL.logger = Postjob.logger = logger
end

SQL = Simple::SQL

# Ensure there is a tmp directory for testing host id file ...
Dir.mkdir("tmp") unless File.exist?("tmp")

RSpec.configure do |config|
  config.run_all_when_everything_filtered = true
  config.filter_run focus: (ENV["CI"] != "true")
  config.expect_with(:rspec) { |c| c.syntax = :expect }
  config.order = "random"
  config.example_status_persistence_file_path = ".rspec.persistence"

  config.before(:all) {}
  config.after {}

  config.before(:each) do
    ::Postjob::Host.clear_storage
  end

  config.before(:each) do
    @worker_session = Postjob.start_worker_session!(heartbeat: false)
  end

  config.after(:each) do
    @worker_session = nil
    Postjob.stop_worker_session!
  end
end
